@@ -45,15 +45,32 @@ static bool userNamespace(pid_t pid);
 static int getX11Display(pid_t pid);
 
 
-StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0),
+// find the first child process for the specified pid
+// return -1 if not found
+static int find_child(int id) {
+	int i;
+	for (i = 0; i < max_pids; i++) {
+		if (pids[i].level == 2 && pids[i].parent == id)
+			return i;
+	}
+
+	return -1;
+}
+
+StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0), lts_(false),
 	pid_initialized_(false), pid_seccomp_(false), pid_caps_(QString("")), pid_noroot_(false),
 	pid_cpu_cores_(QString("")), pid_protocol_(QString("")), pid_name_(QString("")),
 	profile_(QString("")), pid_x11_(0),
-	have_join_(true), caps_cnt_(64), graph_type_(GRAPH_4MIN), no_network_(false) {
+	have_join_(true), caps_cnt_(64), graph_type_(GRAPH_4MIN), net_none_(false) {
 
 	// clean storage area
 	cleanStorage();
 
+	// detect LTS version
+	char *str = run_program("firejail --version");
+	if (str && strstr(str, "LTS"))
+		lts_ = true;
+
 	procView_ = new QTextBrowser;
 	procView_->setOpenLinks(false);
 	procView_->setOpenExternalLinks(false);
@@ -87,7 +104,7 @@ StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0),
 	}
 
 	// detect the number of capabilities supported by the current kernel
-	char *str = run_program("firejail --debug-caps");
+	str = run_program("firejail --debug-caps");
 	if (!str)
 		return;
 	int val;
@@ -97,13 +114,8 @@ StatsDialog::StatsDialog(): QDialog(), mode_(MODE_TOP), pid_(0), uid_(0),
 		caps_cnt_ = val;
 	}
 
-	struct stat s;
-	if (getuid() != 0 && stat("/proc/sys/kernel/grsecurity", &s) == 0)
-		no_network_ = true;
-
 	thread_ = new PidThread();
 	connect(thread_, SIGNAL(cycleReady()), this, SLOT(cycleReady()));
-
 }
 
 StatsDialog::~StatsDialog() {
@@ -115,6 +127,8 @@ void StatsDialog::cleanStorage() {
 	storage_dns_ = "";
 	storage_caps_ = "";
 	storage_seccomp_ = "";
+	storage_intro_ = "";
+	storage_network_ = "";
 }
 
 QString StatsDialog::header() {
@@ -134,9 +148,10 @@ QString StatsDialog::header() {
 			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"shut\">Shutdown</a>";
 		if (have_join_ && uid_ == getuid())
 			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"join\">Join</a>";
-		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"fmgr\">File Manager</a>";
+		if (!lts_)
+			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"fmgr\">File Manager</a>";
 		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"tree\">Process Tree</a>";
-		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"dns\">DNS</a>";
+		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"network\">Network</a>";
 		msg += "</td></tr></table>";
 	}
 
@@ -148,18 +163,19 @@ QString StatsDialog::header() {
 			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"shut\">Shutdown</a>";
 		if (have_join_ && uid_ == getuid())
 			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"join\">Join</a>";
-		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"fmgr\">File Manager</a>";
+		if (!lts_)
+			msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"fmgr\">File Manager</a>";
 		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"tree\">Process Tree</a>";
-		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"dns\">DNS</a>";
+		msg += " &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<a href=\"network\">Network</a>";
 		msg += "</td></tr></table>";
 	}
 
-
+	msg += "<hr>";
 	return msg;
 }
 
 void StatsDialog::updateTop() {
-	QString msg = header() + "<hr>";
+	QString msg = header();
 	msg += "<table><tr><td width=\"5\"></td><td><b>Sandbox List</b></td></tr></table><br/>\n";
 	msg += "<table><tr><td width=\"5\"></td><td width=\"60\">PID</td/><td width=\"60\">CPU<br/>(%)</td><td>Memory<br/>(KiB)&nbsp;&nbsp;</td><td>RX<br/>(KB/s)&nbsp;&nbsp;</td><td>TX<br/>(KB/s)&nbsp;&nbsp;</td><td>Command</td>\n";
 
@@ -199,7 +215,7 @@ void StatsDialog::updateFirewall() {
 
 	if (arg_debug)
 		printf("reading firewall configuration\n");
-	QString msg = header() + "<hr>";
+	QString msg = header() + storage_intro_;
 
 	char *cmd;
 	if (asprintf(&cmd, "firejail --netfilter.print=%d", pid_) != -1) {
@@ -222,8 +238,8 @@ void StatsDialog::updateTree() {
 
 	if (arg_debug)
 		printf("reading process tree configuration\n");
-	QString msg = header();
-	msg += "<hr><table><tr><td width=\"5\"></td><td>";
+	QString msg = header() + storage_intro_;
+	msg += "<table><tr><td width=\"5\"></td><td>";
 
 	char *str = 0;
 	char *cmd;
@@ -265,8 +281,8 @@ void StatsDialog::updateSeccomp() {
 	if (msg.isEmpty()) {
 		if (arg_debug)
 			printf("reading seccomp configuration\n");
-		QString msg = header();
-		msg += "<hr><table><tr><td width=\"5\"></td><td>";
+		QString msg = header() + storage_intro_;
+		msg += "<table><tr><td width=\"5\"></td><td>";
 
 		char *str = 0;
 		char *cmd;
@@ -310,8 +326,8 @@ void StatsDialog::updateCaps() {
 	if (msg.isEmpty()) {
 		if (arg_debug)
 			printf("reading caps configuration\n");
-		msg = header();
-		msg += "<hr><table><tr><td width=\"5\"></td><td>";
+		msg = header() + storage_intro_;
+		msg += "<table><tr><td width=\"5\"></td><td>";
 
 		char *str = 0;
 		char *cmd;
@@ -344,50 +360,261 @@ void StatsDialog::updateCaps() {
 	}
 }
 
-void StatsDialog::updateDns() {
+static QString get_dns(int pid) {
+	QString rv;
+
+	char *str = 0;
+	char *cmd;
+	if (asprintf(&cmd, "firejail --dns.print=%d", pid) != -1) {
+		str = run_program(cmd);
+		char *ptr = str;
+
+		// htmlize!
+		while (*ptr != 0) {
+			if (*ptr == '\n') {
+				*ptr = '\0';
+				bool skip = false;
+				if (*str == '#')
+					skip = true;
+				if (!skip)
+					rv += QString(str) + "<br/>\n";
+				ptr++;
+
+				while (*ptr == ' ') {
+					if (!skip)
+						rv += "&nbsp;&nbsp;";
+					ptr++;
+				}
+				str = ptr;
+				continue;
+			}
+			ptr++;
+		}
+	}
+	free(cmd);
+	return rv;
+}
+
+// build the network interface list for firejail versions 0.9.56 or older, including 0.9.56-LTS
+static QString get_interfaces_old(int pid) {
+	QString rv;
+
+	char *fname;
+	if (asprintf(&fname, "/run/firejail/network/%d-netmap", pid) == -1)
+		errExit("asprintf");
+
+	FILE *fp = fopen(fname, "r");
+	if (fp) {
+		char buf[4096];
+		int i = -1;
+		while (fgets(buf, 4096, fp)) {
+			i++;
+			char *ptr = strchr(buf, '\n');
+			if (ptr)
+				*ptr = '\0';
+
+			// extract parent device
+			ptr = strchr(buf, ':');
+			if (!ptr)
+				continue;
+			char *parent_dev = buf;
+			*ptr = '\0';
+			ptr++;
+			char *child_dev = ptr;
+
+			QString str;
+			str.sprintf("%s (parent device %s", child_dev, parent_dev);
+
+			// detect bridge device
+			char *sysfile;
+			if (asprintf(&sysfile, "/sys/class/net/%s/bridge", parent_dev) == -1)
+				errExit("asprintf");
+			struct stat s;
+			if (stat(sysfile, &s) == 0)
+				str += ", bridge)";
+			else
+				str += ")";
+			free(sysfile);
+
+			rv += str + "<br/>";
+		}
+		fclose(fp);
+	}
+	free(fname);
+
+	return rv;
+}
+
+// build the network interface list for firejail versions 0.9.57 and up
+// returns an empty string if --net.print is not available in the currently installed firejail version
+static QString get_interfaces_new(int pid) {
+	QString rv;
+	char *str = 0;
+	char *cmd;
+	if (asprintf(&cmd, "firejail --net.print=%d 2>&1", pid) != -1) {
+		str = run_program(cmd);
+		free(cmd);
+
+		// htmlize!
+		char *ptr = strtok(str, "\n");
+		if (!ptr || strncmp(ptr, "Error", 5) == 0)
+			goto errexit;
+		while ((ptr = strtok(NULL, "\n")) != NULL) {
+			if (strncmp(ptr, "Error", 5) == 0)
+				goto errexit;
+			if (strncmp(ptr, "Interface ", 10) == 0)
+				continue;
+			if (strncmp(ptr, "lo ", 3) == 0)
+				continue;
+
+			// parse the interface line, example
+			//eth0-12202       c6:7f:d1:a9:3d:bc  192.168.1.82     255.255.255.0    UP
+			// ifname
+			char *ifname = ptr;
+			while (*ptr != ' ' && *ptr != '\0')
+				ptr++;
+			if (*ptr == '\0')
+				goto errexit;
+			*ptr = '\0';
+			ptr++;
+
+			// skip mac address
+			while (*ptr == ' ')
+				ptr++;
+			while (*ptr != ' ' && *ptr != '\0')
+				ptr++;
+			if (*ptr == '\0')
+				goto errexit;
+			while (*ptr == ' ')
+				ptr++;
+
+			// ip address
+			char *ip = ptr;
+			while (*ptr != ' ' && *ptr != '\0')
+				ptr++;
+			if (*ptr == '\0')
+				goto errexit;
+			*ptr = '\0';
+			ptr++;
+			while (*ptr == ' ')
+				ptr++;
+
+			// extract mask...
+			char *mask	= ptr;
+			while (*ptr != ' ' && *ptr != '\0')
+				ptr++;
+			if (*ptr == '\0')
+				goto errexit;
+			*ptr = '\0';
+			// ... and build a CIDR addrss
+			uint32_t mask_uint32;
+			if (atoip(mask, &mask_uint32))
+				goto errexit;
+			int bits = mask2bits(mask_uint32);
+			rv += QString(ifname) + "&nbsp;&nbsp;&nbsp;" + QString(ip) + "/" +
+				QString::number(bits) + "<br/>";
+		}
+	}
+
+	return rv;
+
+errexit:
+	return QString(); // empty string
+}
+
+void StatsDialog::updateNetwork() {
+	int cycle = Db::instance().getCycle();
+	assert(cycle < DbPid::MAXCYCLE);
 	DbPid *dbptr = Db::instance().findPid(pid_);
 	if (!dbptr) {
 		mode_ = MODE_TOP;
 		return;
 	}
 
-	QString msg = storage_dns_;
-	if (msg.isEmpty()) {
+	// DNS
+	QString msg = header() + storage_intro_;
+	if (storage_dns_.isEmpty()) {
 		if (arg_debug)
 			printf("reading dns configuration\n");
 
-		msg = header();
-		msg += "<hr><table><tr><td width=\"5\"></td><td>";
-
-		char *str = 0;
-		char *cmd;
-		if (asprintf(&cmd, "firejail --dns.print=%d", pid_) != -1) {
-			str = run_program(cmd);
-			char *ptr = str;
-
-			// htmlize!
-			while (*ptr != 0) {
-				if (*ptr == '\n') {
-					*ptr = '\0';
-					msg += QString(str) + "<br/>\n";
-					ptr++;
-
-					while (*ptr == ' ') {
-						msg += "&nbsp;&nbsp;";
-						ptr++;
-					}
-					str = ptr;
-					continue;
-				}
-				ptr++;
-			}
-		}
-		free(cmd);
-
-		msg += "</td></tr></table>";
-		procView_->setHtml(msg);
-		storage_dns_ = msg;
+		storage_dns_ += "<table><tr><td width=\"5\"></td><td><b>DNS</b><br/>";
+		storage_dns_ += get_dns(pid_);
+		storage_dns_ += "</td>";
 	}
+	msg += storage_dns_;
+
+	// network interfaces
+	if (storage_network_.isEmpty()) {
+//printf("network namespace disabled %d, net_none_ %d\n", dbptr->networkDisabled(), net_none_);
+		if (net_none_)
+			storage_network_ = "<td><b>Network Interfaces</b><br/>lo<br/>";
+		else if (dbptr->networkDisabled())
+			storage_network_ = "<td>Using the system network namespace";
+		else {
+
+			storage_network_ = "<td><b>Network Interfaces</b><br/>lo<br/>";
+			QString tmp = get_interfaces_new(pid_);
+			if (tmp.isEmpty())
+				tmp = get_interfaces_old(pid_);
+			storage_network_ += tmp;
+		}
+		storage_network_ += "</td></tr>";
+
+	}
+	msg += storage_network_;
+
+
+
+	// graph type
+	msg += "<tr><td></td>";
+	if (dbptr->networkDisabled() == false && net_none_ == false) {
+		if (graph_type_ == GRAPH_4MIN) {
+			msg += "<td><b>Stats: </b>1min <a href=\"1h\">1h</a> <a href=\"12h\">12h</a></td>";
+		}
+		else if (graph_type_ == GRAPH_1H) {
+			msg += "<td><b>Stats: </b><a href=\"1min\">1min</a> 1h <a href=\"12h\">12h</a></td>";
+		}
+		else if (graph_type_ == GRAPH_12H) {
+			msg += "<td><b>Stats: </b><a href=\"1min\">1min</a> <a href=\"1h\">1h</a> 12h</td>";
+		}
+		else
+			assert(0);
+	}
+
+	// netfilter
+	if (dbptr->networkDisabled() == false && net_none_ == false)
+		msg += "<td><b>Firewall</b>: <a href=\"firewall\">enabled</a></td></tr>\n";
+	else
+		msg += "<td><b>Firewall</b>: system firewall</td></tr>\n";
+
+
+	if (dbptr->networkDisabled() == false && net_none_ == false)
+		msg += "<tr><td></td><td>"+ graph(2, dbptr, cycle, graph_type_) + "</td><td>" + graph(3, dbptr, cycle, graph_type_) + "</td></tr>";
+
+	msg += QString("</table><br/>");
+
+	// bandwidth limits
+	if (dbptr->networkDisabled() == false && net_none_ == false) {
+		char *fname;
+		if (asprintf(&fname, "/run/firejail/bandwidth/%d-bandwidth", pid_) == -1)
+			errExit("asprintf");
+		FILE *fp = fopen(fname, "r");
+		if (fp) {
+			msg += "<br/><table><tr><td width=\"5\"></td><td>";
+			msg += "<b>Bandwidth limits:</b><br/><br/>\n";
+			char buf[1024];
+			while (fgets(buf, 1024, fp)) {
+				msg += buf;
+				msg += "<br/>";
+			}
+			fclose(fp);
+			msg += "</td></tr></table>";
+		}
+		free(fname);
+	}
+
+	procView_->setHtml(msg);
+
 }
 
 void StatsDialog::kernelSecuritySettings() {
@@ -478,22 +705,6 @@ void StatsDialog::kernelSecuritySettings() {
 	free(cmd);
 }
 
-
-// find the first child process for the specified pid
-// return -1 if not found
-static int find_child(int id) {
-	int i;
-	for (i = 0; i < max_pids; i++) {
-		if (pids[i].level == 2 && pids[i].parent == id)
-			return i;
-	}
-
-	return -1;
-}
-
-
-
-
 void StatsDialog::updatePid() {
 	QString msg = "";
 
@@ -519,6 +730,25 @@ void StatsDialog::updatePid() {
 		profile_ = getProfile(pid_);
 		pid_x11_ = getX11Display(pid_);
 		pid_initialized_ = true;
+
+		// detect --net=none
+		int child = find_child(pid_);
+		char *fname;
+		if (asprintf(&fname, "/proc/%d/net/dev", child) == -1)
+			errExit("asprintf");
+		FILE *fp = fopen(fname, "r");
+		if (fp) {
+			char buf[4096];
+			int cnt = 0;
+			while (fgets(buf, 4096, fp))
+				cnt++;
+			fclose(fp);
+			if (cnt <= 3)
+				net_none_ = true;
+			else
+				net_none_ = false;
+		}
+		free(fname);
 	}
 
 	// get user name
@@ -528,25 +758,28 @@ void StatsDialog::updatePid() {
 		errExit("getpwuid");
 	uid_ = pw->pw_uid;
 
-	msg += header() + "<hr>";
-	msg += "<table>";
+	// add header
+	msg += header();
+
+	// add intro
+	storage_intro_ = "<table>";
 	if (!pid_name_.isEmpty())
-		msg += "<tr><td width=\"5\"></td><td><b>Sandbox name:</b> " + pid_name_ + "</td></tr>";
-	msg += "<tr><td width=\"5\"></td><td><b>Command:</b> " + QString(cmd) + "</td></tr>";
+		storage_intro_ += "<tr><td width=\"5\"></td><td><b>Sandbox name:</b> " + pid_name_ + "</td></tr>";
+	storage_intro_ += "<tr><td width=\"5\"></td><td><b>Command:</b> " + QString(cmd) + "</td></tr>";
 	if (!profile_.isEmpty())
-		msg += "<tr><td width=\"5\"></td><td><b>Profile:</b> " + profile_ + "</td></tr>";
-	// add
-	msg += "</table><br/>";
+		storage_intro_ += "<tr><td width=\"5\"></td><td><b>Profile:</b> " + profile_ + "</td></tr>";
+	storage_intro_ += "</table><br/>";
+	msg += storage_intro_;
 
 	msg += "<table>";
 	msg += QString("<tr><td width=\"5\"></td><td><b>PID:</b> ") +  QString::number(pid_) + "</td>";
-	if (ptr->networkDisabled() || no_network_)
+	if (ptr->networkDisabled() || net_none_)
 		msg += "<td><b>RX:</b> unknown</td></tr>";
 	else
 		msg += QString("<td><b>RX:</b> ") + QString::number(st->rx_) + " KB/s</td></tr>";
 
 	msg += QString("<tr><td></td><td><b>User:</b> ") + pw->pw_name  + "</td>";
-	if (ptr->networkDisabled() || no_network_)
+	if (ptr->networkDisabled() || net_none_)
 		msg += "<td><b>TX:</b> unknown</td></tr>";
 	else
 		msg += QString("<td><b>TX:</b> ") + QString::number(st->tx_) + " KB/s</td></tr>";
@@ -613,40 +846,12 @@ void StatsDialog::updatePid() {
 	else
 		assert(0);
 
-	// netfilter
-	if (ptr->networkDisabled() == false && no_network_ == false)
-		msg += "<td><b>Firewall</b>: <a href=\"firewall\">enabled</a></td></tr>\n";
-	else
-		msg += "<td><b>Firewall</b>: system firewall</td></tr>\n";
-
 	// graphs
 	msg += "<tr></tr>";
 	msg += "<tr><td></td><td>"+ graph(0, ptr, cycle, graph_type_) + "</td><td>" + graph(1, ptr, cycle, graph_type_) + "</td></tr>";
-	if (ptr->networkDisabled() == false && no_network_ == false)
-		msg += "<tr><td></td><td>"+ graph(2, ptr, cycle, graph_type_) + "</td><td>" + graph(3, ptr, cycle, graph_type_) + "</td></tr>";
 
 	msg += QString("</table><br/>");
 
-	// bandwidth limits
-	if (ptr->networkDisabled() == false && no_network_ == false) {
-		char *fname;
-		if (asprintf(&fname, "/run/firejail/bandwidth/%d-bandwidth", pid_) == -1)
-			errExit("asprintf");
-		FILE *fp = fopen(fname, "r");
-		if (fp) {
-			msg += "<br/><table><tr><td width=\"5\"></td><td>";
-			msg += "<b>Bandwidth limits:</b><br/><br/>\n";
-			char buf[1024];
-			while (fgets(buf, 1024, fp)) {
-				msg += buf;
-				msg += "<br/>";
-			}
-			fclose(fp);
-			msg += "</td></tr></table>";
-		}
-		free(fname);
-	}
-
 	procView_->setHtml(msg);
 }
 
@@ -659,8 +864,8 @@ void StatsDialog::cycleReady() {
 		updateTree();
 	else if (mode_ == MODE_SECCOMP)
 		updateSeccomp();
-	else if (mode_ == MODE_DNS)
-		updateDns();
+	else if (mode_ == MODE_NETWORK)
+		updateNetwork();
 	else if (mode_ == MODE_CAPS)
 		updateCaps();
 	else if (mode_ == MODE_FIREWALL)
@@ -681,7 +886,7 @@ void StatsDialog::anchorClicked(const QUrl & link) {
 			mode_ = MODE_PID;
 		else if (mode_ == MODE_SECCOMP)
 			mode_ = MODE_PID;
-		else if (mode_ == MODE_DNS)
+		else if (mode_ == MODE_NETWORK)
 			mode_ = MODE_PID;
 		else if (mode_ == MODE_CAPS)
 			mode_ = MODE_PID;
@@ -710,8 +915,8 @@ void StatsDialog::anchorClicked(const QUrl & link) {
 	else if (linkstr == "1min") {
 		graph_type_ = GRAPH_4MIN;
 	}
-	else if (linkstr == "dns") {
-		mode_ = MODE_DNS;
+	else if (linkstr == "network") {
+		mode_ = MODE_NETWORK;
 	}
 	else if (linkstr == "firewall") {
 		mode_ = MODE_FIREWALL;
